#!/usr/bin/python -Es
#coding=utf-8
#
# polgengui.py - GUI for SELinux Config tool in sepolicy-tool
#
# Dan Walsh <dwalsh@redhat.com>
#
# Copyright (C) 2007-2011 Red Hat 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import signal
import string
import gtk
import gtk.glade
import os
import gobject
import gnome
import sys
import polgen
import re
import datetime


##
## I18N
## 
PROGNAME="policycoreutils"

import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME,
                    localedir="/usr/share/locale",
                    unicode=False,
                    codeset = 'utf-8')
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

gnome.program_init("SELinux Policy Generation Tool", "5")

version = "1.0"

sys.path.append('/usr/share/sepolicy-tool')
sys.path.append('.')

# From John Hunter http://www.daa.com.au/pipermail/pygtk/2003-February/004454.html
def foreach(model, path, iter, selected):
    selected.append(model.get_value(iter, 0))

##
## Pull in the Glade file
##
if os.access("polgen.glade", os.F_OK):
    xml = gtk.glade.XML ("polgen.glade", domain=PROGNAME)
else:
    xml = gtk.glade.XML ("/usr/share/sepolicy-tool/polgen.glade", domain=PROGNAME)

FILE = 1
DIR = 2

def info(message):
	dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
			gtk.BUTTONS_OK,
			message)
	dlg.set_position(gtk.WIN_POS_CENTER)
	dlg.show_all()
	rc=dlg.run()
	dlg.destroy()
	return rc


class childWindow:
    #START_PAGE = 0
    #SELECT_TYPE_PAGE = 0
    #SELECT_NAME_PAGE = 1
    #SELECT_INSTALL_TYPE_PAGE = 2
    #SELECT_DIRS_FILES_PAGE= 3
    #USER_TRANSITION_PAGE = 4
    #IN_NET_PAGE = 5
    #OUT_NET_PAGE = 6
    #COMMON_APPS_PAGE = 7
    #SELECT_POLGEN_DIR_PAGE = 8
    #FINISH_PAGE = 8

    START_PAGE = 0
    SELECT_NAME_PAGE = 0
    SELECT_INSTALL_TYPE_PAGE = 1
    SELECT_DIRS_FILES_PAGE= 2
    USER_TRANSITION_PAGE = 3
    SELECT_POLGEN_DIR_PAGE = 4
    FINISH_PAGE = 4

    def __init__(self):
        self.xml = xml
        self.notebook = xml.get_widget ("notebook")
        self.storefilesnotebook = xml.get_widget ("store_files_notebook")
        self.info_notebook = xml.get_widget("info_noetbook")
	self.app_radiobutton = xml.get_widget("app_radiobutton")
	self.database_radiobutton = xml.get_widget("database_radiobutton")
	self.midware_radiobutton = xml.get_widget("midware_radiobutton")
	self.sys_assembly_radiobutton = xml.get_widget("sys_assembly_radiobutton")
	self.rpm_radiobutton = xml.get_widget("rpm_radiobutton")
	self.advance_checkbutton = xml.get_widget("advance_checkbutton")
	self.advance_checkbutton.connect("toggled",self.advance_checkbutton_toggled)
	self.advance_checkbutton.set_sensitive(0)
	self.rpm_radiobutton.connect("toggled",self.rpm_radiobutton_toggled)
		

	self.expander = xml.get_widget("expander1")	
	self.expander.set_sensitive(0)

	self.prb = xml.get_widget("progressbar1")

        self.label_dict = {}
        self.tooltip_dict = {}

        label = xml.get_widget ("select_install_type_label")
        self.label_dict[label] = label.get_text()

        label = xml.get_widget ("select_dirs_files_label")
        self.label_dict[label] = label.get_text()

        label = xml.get_widget ("select_user_roles_label")
        self.label_dict[label] = label.get_text()

        label = xml.get_widget ("select_polgen_dir_label")
        self.label_dict[label] = label.get_text()

        label = xml.get_widget ("in_tcp_all_checkbutton")

        label = xml.get_widget ("in_tcp_entry")

        label = xml.get_widget ("in_udp_entry")

        label = xml.get_widget ("out_tcp_entry")

        label = xml.get_widget ("out_udp_entry")

        label = xml.get_widget ("out_tcp_all_checkbutton")

        label = xml.get_widget ("out_udp_all_checkbutton")

        label = xml.get_widget ("exec_treeview")
        self.tooltip_dict[label] = label.get_tooltip_text()

        label = xml.get_widget ("lib_treeview")
        self.tooltip_dict[label] = label.get_tooltip_text()

        label = xml.get_widget ("conf_treeview")
        self.tooltip_dict[label] = label.get_tooltip_text()

        label = xml.get_widget ("log_treeview")
        self.tooltip_dict[label] = label.get_tooltip_text()

        label = xml.get_widget ("tmp_treeview")
        self.tooltip_dict[label] = label.get_tooltip_text()

        try:
            self.all_types = polgen.get_all_types()
            self.all_modules = polgen.get_all_modules()
            self.all_roles = polgen.get_all_roles()
            self.all_users = polgen.get_all_users()
        except RuntimeError, e:
            self.all_types = []
            self.all_modules = []
            self.all_roles = []
            self.all_users = []
            self.error(str(e))

        self.name=""
        xml.signal_connect("on_delete_clicked", self.delete)
        xml.signal_connect("on_init_script_select_clicked", self.init_script_select)

        xml.signal_connect("on_install_master_button_clicked", self.get_install_master_select)

        xml.signal_connect("on_add_file_clicked", self.add_file)
        xml.signal_connect("on_add_dir_clicked", self.add_dir)
        xml.signal_connect("on_about_clicked", self.on_about_clicked)
        xml.get_widget ("cancel_button").connect("clicked",self.quit)	
        self.forward_button = xml.get_widget ("forward_button")	
        self.forward_button.connect("clicked",self.forward)		
        self.back_button = xml.get_widget ("back_button")
        self.back_button.connect("clicked",self.back)
	
	self.rpm_scan_button = self.xml.get_widget("rpm_scan_button")
	self.rpm_scan_button.connect("clicked",self.rpm_file_select)

        self.pages={}

        for i in polgen.APPLICATIONS:
            self.pages[i] = [  self.SELECT_NAME_PAGE, self.SELECT_DIRS_FILES_PAGE, self.SELECT_POLGEN_DIR_PAGE]
#        import pdb
        #pdb.set_trace()
        self.pages[polgen.USER] = [  self.SELECT_NAME_PAGE, self.SELECT_INSTALL_TYPE_PAGE, self.SELECT_DIRS_FILES_PAGE, self.USER_TRANSITION_PAGE,  self.SELECT_POLGEN_DIR_PAGE ]


        self.current_page = 0
        self.back_button.set_sensitive(0)

        self.network_buttons = {}

        self.in_tcp_all_checkbutton = xml.get_widget ("in_tcp_all_checkbutton")
        self.in_tcp_entry = self.xml.get_widget("in_tcp_entry")
        self.network_buttons[self.in_tcp_all_checkbutton] = [ self.in_tcp_entry ]


        self.out_tcp_all_checkbutton = xml.get_widget ("out_tcp_all_checkbutton")
        self.out_tcp_entry = self.xml.get_widget("out_tcp_entry")

        self.network_buttons[self.out_tcp_all_checkbutton] = [ self.out_tcp_entry ]

        self.in_udp_all_checkbutton = xml.get_widget ("in_udp_all_checkbutton")
        self.in_udp_entry = self.xml.get_widget("in_udp_entry")

        self.network_buttons[self.in_udp_all_checkbutton] = [ self.in_udp_entry ]

        self.out_udp_all_checkbutton = xml.get_widget ("out_udp_all_checkbutton")
        self.out_udp_entry = self.xml.get_widget("out_udp_entry")
        self.network_buttons[self.out_udp_all_checkbutton] = [ self.out_udp_entry ]

	self.page_fixed1 = self.xml.get_widget("fixed3")
	self.page_fixed1.connect("expose-event",self.draw_pixbuf_one)
   
	self.page_fixed2 = self.xml.get_widget("fixed4")
	self.page_fixed2.connect("expose-event",self.draw_pixbuf_two)

	self.page_fixed3 = self.xml.get_widget("fixed5")
	self.page_fixed3.connect("expose-event",self.draw_pixbuf_three)

	self.page_fixed4 = self.xml.get_widget("fixed6")
	self.page_fixed4.connect("expose-event",self.draw_pixbuf_four)

	self.page_fixed5 = self.xml.get_widget("fixed10")
	self.page_fixed5.connect("expose-event",self.draw_pixbuf_five)

        self.roles = []

        for b in self.network_buttons.keys():
            b.connect("clicked",self.network_all_clicked)

	
	
    def confine_application(self):
        return self.get_type() in polgen.APPLICATIONS

#点击前进按钮后执行的函数
    def forward(self, arg):
        type = self.get_type()
        if self.current_page == self.START_PAGE:
            self.back_button.set_sensitive(1)


        if self.pages[type][self.current_page] == self.SELECT_NAME_PAGE:
            if self.on_name_page_next():
                return

        if self.pages[type][self.current_page] == self.SELECT_INSTALL_TYPE_PAGE:
	####当rpm包选择按钮激活时，执行的动作
	    if self.rpm_radiobutton.get_active():
	    	self.file_auto_store(self.rpm_file_list())

	####当应用程序安装目录按钮激活时，执行的动作
	    if self.install_other_radiobutton.get_active(): 
		install_path = self.install_master_entry.get_text()
		if (not os.path.isfile(install_path.strip())) and  (not os.path.isdir(install_path.strip())):
			self.error(_("请输入正确的文件或是目录名"))
			return
#=====================ADD BY LIUSIHUA 20141104==================================
		if (" " in install_path.strip()):
			self.error(_("目录名中不得含有空格"))
                        return

	
		if install_path.strip().startswith("/tmp"): 
			self.error(_("请不要将软件安装在/tmp目录"))
			return 

		if install_path.strip().startswith("/etc"):
			self.error(_("请不要将软件安装在/etc目录"))
			return 

		if install_path.strip().startswith("/home"):
			self.error(_("请不要将软件安装在/home目录"))
			return

	    	self.parse_install_path()
		
            if self.on_install_type_page_next():
                return

        if self.pages[type][self.current_page] == self.SELECT_POLGEN_DIR_PAGE:
#           outputdir = self.output_entry.get_text()
            outputdir = os.getcwd()
            if not os.path.isdir(outputdir):
                self.error(_("%s must be a directory") % outputdir )
                return False
	
        #if self.pages[type][self.current_page] == self.SELECT_DIRS_FILES_PAGE:
	#    if BINFILENU == 0:
	#	self.error(_("您的程序中没有可执行文件"))
	#	return 

        if self.pages[type][self.current_page] == self.FINISH_PAGE:
            self.generate_policy()
            self.xml.get_widget ("cancel_button").set_label(gtk.STOCK_CLOSE)
        else:
            self.current_page = self.current_page + 1
            self.notebook.set_current_page(self.pages[type][self.current_page])
            if self.pages[type][self.current_page] == self.FINISH_PAGE:
                self.forward_button.set_label(gtk.STOCK_APPLY)
    ####点击回退按钮执行的动作    
    def back(self,arg):
        type = self.get_type()
        if self.pages[type][self.current_page] == self.FINISH_PAGE:
            self.forward_button.set_label(gtk.STOCK_GO_FORWARD)

        self.current_page = self.current_page - 1 
        self.notebook.set_current_page(self.pages[type][self.current_page])
        if self.pages[type][self.current_page] == self.START_PAGE:
            self.back_button.set_sensitive(0)
	
	if self.pages[type][self.current_page] ==  1:
		if self.rpm_radiobutton.get_active() and self.rpm_entry.get_text() != None:
			self.rpm_entry.set_text("")
		self.exec_store.clear()
		self.lib_store.clear()
		self.conf_store.clear()
		self.log_store.clear()
		self.tmp_store.clear()
        
    def network_all_clicked(self, button):
        active = button.get_active()
        for b in self.network_buttons[button]:
            b.set_sensitive(not active)
        
    def verify(self, message, title="" ):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_YES_NO,
                                message)
        dlg.set_title(title)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc

#####弹出的用户信息界面
    def info(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_OK,
                                message)
        dlg.set_position(gtk.WIN_POS_CENTER)
        dlg.show_all()
        rc=dlg.run()
        dlg.destroy()
	return rc

####弹出出错信息界面
    def error(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR,
                                gtk.BUTTONS_CLOSE,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        dlg.run()
        dlg.destroy()

####获得模块名字
    def get_name(self):
        return self.name_entry.get_text()

####获得类型名字
    def get_type(self):
        #if self.user_radiobutton.get_active():
         return polgen.USER
        #if self.init_radiobutton.get_active():
        #    return polgen.DAEMON

    def get_install_master_select(self,*args):
       # if self.install_opt_radiobutton.get_active():
       #     self.install_opt_select()
       # if self.install_usr_local_radiobutton.get_active():
       #     self.install_usr_local_select()
       # if self.install_home_radiobutton.get_active():
       #     self.install_home_select()
        if self.install_other_radiobutton.get_active():
            self.install_other_select()

###显示第一张背景图
    def draw_pixbuf_one(self, widget, event):
	path = '/usr/share/sepolicy-tool/image/create_mod_1.jpg'
	pixbuf = gtk.gdk.pixbuf_new_from_file(path)
	widget.window.draw_pixbuf(widget.style.bg_gc[gtk.STATE_NORMAL], pixbuf, 0, 0,0,0)

###显示第二张背景图
    def draw_pixbuf_two(self, widget, event):
	path = '/usr/share/sepolicy-tool/image/create_mod_2.jpg'
	pixbuf = gtk.gdk.pixbuf_new_from_file(path)
	widget.window.draw_pixbuf(widget.style.bg_gc[gtk.STATE_NORMAL], pixbuf, 0, 0,0,0)

###显示第三张背景图
    def draw_pixbuf_three(self, widget, event):
	path = '/usr/share/sepolicy-tool/image/create_mod_3.png'
	pixbuf = gtk.gdk.pixbuf_new_from_file(path)
	widget.window.draw_pixbuf(widget.style.bg_gc[gtk.STATE_NORMAL], pixbuf, 0, 0,0,0)
	
###显示第四张背景图
    def draw_pixbuf_four(self, widget, event):
	path = '/usr/share/sepolicy-tool/image/create_mod_4.jpg'
	pixbuf = gtk.gdk.pixbuf_new_from_file(path)
	widget.window.draw_pixbuf(widget.style.bg_gc[gtk.STATE_NORMAL], pixbuf, 0, 0,0,0)

###显示第五张背景图
    def draw_pixbuf_five(self, widget, event):
	path = '/usr/share/sepolicy-tool/image/create_mod_5.png'
	pixbuf = gtk.gdk.pixbuf_new_from_file(path)
	widget.window.draw_pixbuf(widget.style.bg_gc[gtk.STATE_NORMAL], pixbuf, 0, 0,0,0)

###点击"高级"按钮，触发的动作
    def advance_checkbutton_toggled(self,args):
	if not self.advance_checkbutton.get_active():
		self.expander.set_sensitive(0)
	else:
		self.expander.set_sensitive(1)

    def advance_option(self):
	if not self.advance_checkbutton.get_active():
		self.in_tcp_all_checkbutton.set_active(True)	
		self.in_udp_all_checkbutton.set_active(True)	
		self.out_tcp_all_checkbutton.set_active(True)	
		self.out_udp_all_checkbutton.set_active(True)	

###rpm包选择按钮和安装路径选择按钮切换触发的动作
    def rpm_radiobutton_toggled(self,args):
	if self.rpm_radiobutton.get_active():	
		self.install_other_radiobutton.set_active(False)
		self.install_master_entry.set_sensitive(0)
		self.rpm_entry.set_sensitive(1)
		self.rpm_scan_button.set_sensitive(1)
		self.install_master_button.set_sensitive(0)
		
	if self.install_other_radiobutton.get_active():
		self.install_other_radiobutton.set_active(True)
		self.install_master_entry.set_sensitive(1)
		self.rpm_entry.set_sensitive(0)
		self.rpm_scan_button.set_sensitive(0)
		self.install_master_button.set_sensitive(1)

	
		
###列出一个rpm包含的所有文件和目录
    def rpm_file_list(self):
	fullname = self.rpm_entry.get_text()
	rpm_pagname = fullname.rstrip('.rpm')
	if self.rpm_radiobutton.get_active():
		if (os.system('rpm -q'+" "+rpm_pagname) != 0) or rpm_pagname == None :
			self.error(_("请选择正确的rpm包!"))
			return 
	fd = os.popen('rpm -ql'+" "+rpm_pagname)
	lines = fd.readlines()
	fd.close()
	return lines

###点击rpm包浏览按钮，所弹出的框
    def rpm_file_select(self,args):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("请选择一个rpm包"))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return

	if rc == -4:
		self.rpm_entry.set_text("")
		return 

	self.rpm_entry.set_text(os.path.basename(self.file_dialog.get_filename()))

###用户角色存储
    def role_to_store(self):
	role_user ={"user_r":"user","sysadm_r":"root","secadm_r":"secadm","auditadm_r":"auditadm"}
	for i in self.all_roles:
		if i in role_user.keys():
			iter = self.role_store.append()
			self.role_store.set_value(iter, 1, i.strip())
			self.role_store.set_value(iter, 2,role_user[i.strip()])

###点击具体角色前的选择框时，所触发的动作
    def role_toggled(self,widget,row):
        iter = self.role_store.get_iter(row)	

	if self.role_store.get_value(iter,0) == True:
		self.role_store.set_value(iter,0,False)
	else:
		self.role_store.set_value(iter,0,True)

###将文件添加到liststore中
    def file_auto_store(self,lines):
	for file_dir in lines:
		###当文件名为系统文件时，不将这些文件加入liststore中
		if (file_dir.strip() == "/usr/bin") or (file_dir.strip() == "/usr/lib") or \
			(file_dir.strip() == "/home") or (file_dir.strip() == "/bin") or \
			(file_dir.strip() == "/dev") or (file_dir.strip() == "/lib") or \
			(file_dir.strip() == "/lib64") or (file_dir.strip() == "/etc") or \
			(file_dir.strip() == "/lib/modules") or (file_dir.strip() == "/usr/share/applications") or \
			(file_dir.strip() == "/usr/share/pixmaps") or (file_dir.strip() == "/usr/share/mime/packages"): 
			continue 

		if ("init.d" in file_dir) or ("icons" in file_dir) or ("template" in file_dir) or (".png" in file_dir):
			continue
		
		if ("bin"  in file_dir ) and os.path.isfile(file_dir.strip()) and \
			os.access(file_dir.strip(),os.X_OK):
			exec_iter = self.exec_store.append()
			self.exec_store.set_value(exec_iter, 0, file_dir.strip())
			self.exec_store.set_value(exec_iter, 1, 1)
		elif ("bin"  in file_dir) and os.path.isdir(file_dir.strip()):
			exec_iter = self.exec_store.append()
			self.exec_store.set_value(exec_iter,0,file_dir.strip())
			self.exec_store.set_value(exec_iter,1,2)
			continue 

		elif ("lib" in file_dir or "include" in file_dir ) and os.path.isfile(file_dir.strip()):
			lib_iter = self.lib_store.append()
                        self.lib_store.set_value(lib_iter, 0, file_dir.strip())
                        self.lib_store.set_value(lib_iter, 1, 1)		
		elif ("lib" in file_dir or "include" in file_dir ) and os.path.isdir(file_dir.strip()) :
			list_tmp = file_dir.strip().split('/')
			if list_tmp[-1]  == "include":
				continue 
	
			lib_iter = self.lib_store.append()
                        self.lib_store.set_value(lib_iter, 0, file_dir.strip())
                        self.lib_store.set_value(lib_iter, 1, 2)
#		elif ("include" in file_dir ) and os.path.isdir(file_dir.strip()):
	#		for root,dir,files in os.walk(file_dir.strip()):
	#			print  file_dir
	#			for filename in files:
	#				lib_iter = self.lib_store.append()
	#				self.lib_store.set_value(lib_iter,0,file_dir+os.sep+filename.strip())
	#				self.lib_store.set_value(lib_iter, 1, 1)
			continue


		elif ("conf" in file_dir or "etc" in file_dir ) and os.path.isfile(file_dir.strip()) :
			conf_iter = self.conf_store.append()
			self.conf_store.set_value(conf_iter,0,file_dir.strip())
			self.conf_store.set_value(conf_iter,1,1)
		elif ("conf" in file_dir or "etc" in file_dir ) and os.path.isdir(file_dir.strip()):
			conf_iter = self.conf_store.append()
			self.conf_store.set_value(conf_iter,0,file_dir.strip())
			self.conf_store.set_value(conf_iter,1,2)
			continue

		elif ("log" in file_dir) and os.path.isfile(file_dir.strip()) and \
		(not os.access(file_dir.strip(),os.X_OK)):
			log_iter = self.log_store.append()
			self.log_store.set_value(log_iter,0,file_dir.strip())
			self.log_store.set_value(log_iter,1,1)
		elif ("log" in file_dir) and os.path.isdir(file_dir.strip()):
			log_iter = self.log_store.append()
			self.log_store.set_value(log_iter,0,file_dir.strip())
			self.log_store.set_value(log_iter,1,2)
			continue

		else:
			if  os.path.isfile(file_dir.strip()) :
				tmp_iter = self.tmp_store.append()
				self.tmp_store.set_value(tmp_iter,0,file_dir.strip())
				self.tmp_store.set_value(tmp_iter,1,1)
			if  os.path.isdir(file_dir.strip()):
				tmp_iter = self.tmp_store.append()
				self.tmp_store.set_value(tmp_iter,0,file_dir.strip())
				self.tmp_store.set_value(tmp_iter,1,2)

###分析用户输入的安装路径或是通过浏览框选择的路径
    def parse_install_path(self):
	install_path = self.install_master_entry.get_text()
	if self.install_other_radiobutton.get_active():
		if (not os.path.isfile(install_path.strip())) and  (not os.path.isdir(install_path.strip())):
			self.error(_("请输入正确的文件或是目录名"))
			return
	#	if install_path.strip().startswith("/tmp"): 
	#		self.error(_("请不要装软件安装在/tmp目录"))
	#		return 
	#	if install_path.strip().startswith("/etc"):
	#		self.error(_("请不要装软件安装在/etc目录"))
	#		return 

####当用户输入文件时执行的操作
	if os.path.isfile(install_path.strip()):
		exec_iter = self.exec_store.append()
		self.exec_store.set_value(exec_iter,0,install_path.strip())
		self.exec_store.set_value(exec_iter,1,1)

####当用户输入目录时执行的操作
	if os.path.isdir(install_path.strip()):
		lines=os.listdir(install_path)
		for file_dir in lines:
			full_name=install_path.rstrip("/")+os.sep+file_dir
			
#=================================ADD BY LIUSIHUA 20141103================================================
			fd = os.popen('file'+" "+full_name)
        		lines = fd.readlines()

			if (" " in full_name):
	                        continue

			if (("bin" in full_name ) and \
			os.path.isfile(full_name.strip()) and os.access(full_name.strip(),os.X_OK)) or \
			( lines[0].find("executable") != -1 ):
				exec_iter = self.exec_store.append()
				self.exec_store.set_value(exec_iter, 0, full_name.strip())
				self.exec_store.set_value(exec_iter, 1, 1)
			elif ("bin"  in full_name) and os.path.isdir(full_name.strip()):
				exec_iter = self.exec_store.append()
				self.exec_store.set_value(exec_iter,0,full_name.strip())
				self.exec_store.set_value(exec_iter,1,2)
				continue 

			elif ("lib" in full_name or "include" in full_name) and \
			os.path.isfile(full_name.strip()) :
				lib_iter = self.lib_store.append()
				self.lib_store.set_value(lib_iter, 0, full_name.strip())
				self.lib_store.set_value(lib_iter, 1, 1)		
			elif ("lib" in full_name ) and os.path.isdir(full_name.strip()) :
				lib_iter = self.lib_store.append()
				self.lib_store.set_value(lib_iter, 0, full_name.strip())
				self.lib_store.set_value(lib_iter, 1, 2)
			elif ("include" in full_name) and os.path.isdir(full_name.strip()):
				for root,dir,files in os.walk(full_name.strip()):
					for filename in files:
						lib_iter = self.lib_store.append()
						self.lib_store.set_value(lib_iter,0,full_name+os.sep+filename.strip())
						self.lib_store.set_value(lib_iter, 1, 1)		
				continue

			elif ("conf" in full_name) and os.path.isfile(full_name.strip()) :
				conf_iter = self.conf_store.append()
				self.conf_store.set_value(conf_iter,0,full_name.strip())
				self.conf_store.set_value(conf_iter,1,1)
			elif ("conf" in full_name) and os.path.isdir(full_name.strip()):
				conf_iter = self.conf_store.append()
				self.conf_store.set_value(conf_iter,0,full_name.strip())
				self.conf_store.set_value(conf_iter,1,2)
				continue

			elif ("log" in full_name) and os.path.isfile(full_name.strip()) :
				log_iter = self.log_store.append()
				self.log_store.set_value(log_iter,0,full_name.strip())
				self.log_store.set_value(log_iter,1,1)
			elif ("log" in full_name) and os.path.isdir(full_name.strip()):
				log_iter = self.log_store.append()
				self.log_store.set_value(log_iter,0,full_name.strip())
				self.log_store.set_value(log_iter,1,2)
				continue

			else:
			        if os.path.isfile(full_name.strip()):
					tmp_iter = self.tmp_store.append()
					self.tmp_store.set_value(tmp_iter,0,full_name.strip())
					self.tmp_store.set_value(tmp_iter,1,1)
				if os.path.isdir(full_name.strip()):
					tmp_iter = self.tmp_store.append()
					self.tmp_store.set_value(tmp_iter,0,full_name.strip())
					self.tmp_store.set_value(tmp_iter,1,2)
    def handleSpace(self,args):
        m=re.compile(r'\s')
        return m.sub('\\s',args.strip())
###开始生成策略
    def generate_policy(self, *args):
	try:
	    my_policy=polgen.policy(self.get_name(), self.get_type())

	    if self.get_type() in polgen.APPLICATIONS:
		    exec_iter = self.exec_store.get_iter_first()
		    program = self.exec_store.get_value(exec_iter,0)
		    my_policy.set_program(program)
		    my_policy.gen_symbols()

		    my_policy.set_use_syslog(1)
		    my_policy.set_use_tmp(1)
		    my_policy.set_use_uid(1)
		    my_policy.set_use_pam(1)

		    my_policy.set_use_dbus(1)
		    my_policy.set_use_audit(1)
		    my_policy.set_use_terminal(1)
		    my_policy.set_use_mail(1)

		    my_policy.set_use_enforce(self.enforce_radiobutton.get_active() == 1)

####可以在这里将init.d文件名做为参数
                    if self.get_type() is polgen.DAEMON:
                  	my_policy.set_init_script(self.init_script_entry.get_text())

                    if self.get_type() == polgen.USER:
			  if not self.rpm_radiobutton.get_active():
				my_policy.add_dir(self.install_master_entry.get_text())
			
			  role_iter = self.role_store.get_iter_first()
			  while(role_iter):
 				if self.role_store.get_value(role_iter, 0) == True:
					role = self.role_store.get_value(role_iter,1)
					self.roles.append(re.sub("_r","",role))
                    			#self.roles.append(role.rstrip("_r"))
                		role_iter= self.role_store.iter_next(role_iter)
		   
			  #store, iter = self.role_view.get_selection().get_selected()
                          #role = store.get_value(iter,1)
			  #self.roles.append(role.rstrip("_r"))
			 

			  my_policy.set_transition_users(self.roles)

			  #if self.role_sysadm_radiobutton.get_active():
			  #	self.roles.append("sysadm")
			  #	my_policy.set_transition_users(self.roles)
			  #elif self.role_secadm_radiobutton.get_active():
			  #	self.roles.append("secadm")
			  #	my_policy.set_transition_users(self.roles)
			  #elif self.role_auditadm_radiobutton.get_active():
			  #	self.roles.append("auditadm")
			  #	my_policy.set_transition_users(self.roles)
			  #elif self.role_user_radiobutton.get_active():
			  #	self.roles.append("user")
			  #	my_policy.set_transition_users(self.roles)

####当高级选项选中时，执行的动作	
	    self.advance_option()

            my_policy.set_in_tcp(self.in_tcp_all_checkbutton.get_active(), 0, 0, self.in_tcp_entry.get_text())
            my_policy.set_in_udp(self.in_udp_all_checkbutton.get_active(), 0, 0, self.in_udp_entry.get_text())

            my_policy.set_out_tcp(self.out_tcp_all_checkbutton.get_active(), self.out_tcp_entry.get_text())
            my_policy.set_out_udp(self.out_udp_all_checkbutton.get_active(), self.out_udp_entry.get_text())

	    #####add by hsw 20140105

                
            #my_policy.add_dir(self.install_master_entry.get_text())
#####将liststore中存储的内容提取出来，用于生成策略模块

            exec_iter= self.exec_store.get_iter_first()
            while(exec_iter):
                if self.exec_store.get_value(exec_iter, 1) == FILE:
                    my_policy.add_exec_file(self.handleSpace(self.exec_store.get_value(exec_iter, 0)))
                else:
                    my_policy.add_exec_dir(self.handleSpace(self.exec_store.get_value(exec_iter, 0)))
                exec_iter= self.exec_store.iter_next(exec_iter)

            lib_iter= self.lib_store.get_iter_first()
            while(lib_iter):
                if self.lib_store.get_value(lib_iter, 1) == FILE:
                    my_policy.add_lib_file(self.handleSpace(self.lib_store.get_value(lib_iter, 0)))
                else:
                    my_policy.add_lib_dir(self.handleSpace(self.lib_store.get_value(lib_iter, 0)))
                lib_iter= self.lib_store.iter_next(lib_iter)

            conf_iter= self.conf_store.get_iter_first()
            while(conf_iter):
                if self.conf_store.get_value(conf_iter, 1) == FILE:
                    my_policy.add_conf_file(self.handleSpace(self.conf_store.get_value(conf_iter, 0)))
                else:
                    my_policy.add_conf_dir(self.handleSpace(self.conf_store.get_value(conf_iter, 0)))
                conf_iter= self.conf_store.iter_next(conf_iter)

            log_iter= self.log_store.get_iter_first()
            while(log_iter):
                if self.log_store.get_value(log_iter, 1) == FILE:
                    my_policy.add_log_file(self.handleSpace(self.log_store.get_value(log_iter, 0)))
                else:
                    my_policy.add_log_dir(self.handleSpace(self.log_store.get_value(log_iter, 0)))
                log_iter= self.log_store.iter_next(log_iter)

            tmp_iter= self.tmp_store.get_iter_first()
            while(tmp_iter):
                if self.tmp_store.get_value(tmp_iter, 1) == FILE:
                    my_policy.add_tmp_file(self.handleSpace(self.tmp_store.get_value(tmp_iter,0)))
                else:
                    my_policy.add_tmp_dir(self.handleSpace(self.tmp_store.get_value(tmp_iter, 0)))
                tmp_iter= self.tmp_store.iter_next(tmp_iter)

	    	
	    ###########add by hsw 20130704#################
            #iter= self.store.get_iter_first()
            #while(iter):
            #    if self.store.get_value(iter, 1) == FILE:
            #        my_policy.add_file(self.store.get_value(iter, 0))
            #    else:
            #        my_policy.add_dir(self.store.get_value(iter, 0))
            #    iter= self.store.iter_next(iter)
            ###############################################

##策略工具在/tmp目录下为每个应用创建一个目录	    
	    policy = self.get_name()
	    if not os.path.isdir('/tmp/make_policy'):
		os.mkdir('/tmp/make_policy')
	    if not os.path.isdir('/tmp/make_policy/'+policy):
		os.mkdir('/tmp/make_policy/'+policy)
	    os.chdir('/tmp/make_policy/'+policy)
#===去掉高级选项的功能
	    #if (not self.in_tcp_all_checkbutton.get_active()) or (not self.in_udp_all_checkbutton.get_active):
		#if not os.path.isdir("/tmp/make_policy/"+policy+"/port"):
		#	os.system("mkdir /tmp/make_policy/"+policy+"/port")
		#if (not self.in_tcp_all_checkbutton.get_active()) and (self.in_tcp_entry.get_text() !=""):
		#	os.system("echo"+" "+self.in_tcp_entry.get_text()+" "+"> /tmp/make_policy/"+policy+"/port/tcp_in")  
		#if (not self.in_udp_all_checkbutton.get_active()) and (self.in_udp_entry.get_text() !=""):
		#	os.system("echo"+" "+self.in_udp_entry.get_text()+" "+"> /tmp/make_policy/"+policy+"/port/udp_in")  
	
	    #if (not self.out_tcp_all_checkbutton.get_active()) or (not self.out_udp_all_checkbutton.get_active):
		#if not os.path.isdir("/tmp/make_policy/"+policy+"/port"):
		#	os.system("mkdir /tmp/make_policy/"+policy+"/port")
		#if (not self.out_tcp_all_checkbutton.get_active()) and (self.out_tcp_entry.get_text() !=""):
		#	os.system("echo"+" "+self.in_tcp_entry.get_text()+" "+"> /tmp/make_policy/"+policy+"/port/tcp_out")  
		#if (not self.out_udp_all_checkbutton.get_active()) and (self.out_udp_entry.get_text() !=""):
		#	os.system("echo"+" "+self.in_udp_entry.get_text()+" "+"> /tmp/make_policy/"+policy+"/port/udp_out") 
 
            #self.info(my_policy.generate(os.getcwd()))
            my_policy.generate(os.getcwd())


	    policy_sh = policy +".sh"
	    exec_policy = "policy.sh"

	    mod_name = policy
	    mod_ver = "1.0.0"

	    os.rename(policy_sh,exec_policy)
	    POLICY_LOG="/var/policy_log/policy.log"
	    if not os.path.isfile(POLICY_LOG):
		os.system('mkdir /var/policy_log/')
		os.system('touch /var/policy_log/policy.log')

	    os.system('echo "#########################################">>/var/policy_log/policy.log')	
	    os.system('echo $(date) >>/var/policy_log/policy.log 2>&1')

 #=========当配置策略类型为rpm文件时，删除.te文件中的manage_files_pattern(USER_t,TEMPLATETYPE_file_t,TEMPLATETYPE_file_t)
	    if self.rpm_radiobutton.get_active():
	    	os.system("sed -i '/file_t/d'"+" "+policy+".te")

	    os.system('./policy.sh >>/var/policy_log/policy.log 2>&1')

	    create_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
	    install_path = self.install_master_entry.get_text()

###生成策略创建日志
	    os.chdir('/tmp')
	    self.FileHandle= open('installed.log','a'); 
	    self.FileHandle.write(mod_name)
	    self.FileHandle.write('\t')
	    
	    self.FileHandle.write(mod_ver)
	    self.FileHandle.write('\t')

	    self.FileHandle.write(install_path)
	    self.FileHandle.write('\t')
	    
	    if self.app_radiobutton.get_active():
                mod_type = "应用程序"
            if self.database_radiobutton.get_active():
                mod_type = "数据库"
            if self.midware_radiobutton.get_active():
                mod_type = "中间件"
            if self.sys_assembly_radiobutton.get_active():
                mod_type = "系统组件" 
 
	    self.FileHandle.write(mod_type)
	    self.FileHandle.write('\t')

	    self.FileHandle.write(create_time)
	    self.FileHandle.write('\t')
	    self.FileHandle.write('\n')
	    self.FileHandle.close()

	   
	    self.info(self.mod_exit(policy))
	   
	    #当策略模块成功加载到系统中，退出主应用程序
	    if policy in self.all_modules:
	    	gtk.main_quit() 
	    
            return False
        except ValueError, e:
            self.error(e.message)

###模块生成结果弹出框
    def mod_exit(self,args):
	self.all_modules = polgen.get_all_modules()
	if args in self.all_modules:
		ret = args+"生成并成功加载"
	else:
		ret = args+"加载失败"
	return ret
        
    def delete(self, args):
        #store, iter = self.view.get_selection().get_selected()
        #if iter != None:
        #    store.remove(iter)
        #    self.view.get_selection().select_path ((0,))

        exec_store, exec_iter = self.exec_view.get_selection().get_selected()
        if exec_iter != None:
            exec_store.remove(exec_iter)
            self.exec_view.get_selection().select_path ((0,))

        lib_store, lib_iter = self.lib_view.get_selection().get_selected()
        if lib_iter != None:
            lib_store.remove(lib_iter)
            self.lib_view.get_selection().select_path ((0,))

        conf_store, conf_iter = self.conf_view.get_selection().get_selected()
        if conf_iter != None:
            conf_store.remove(conf_iter)
            self.conf_view.get_selection().select_path ((0,))

        log_store, log_iter = self.log_view.get_selection().get_selected()
        if log_iter != None:
            log_store.remove(log_iter)
            self.log_view.get_selection().select_path ((0,))

        tmp_store, tmp_iter = self.tmp_view.get_selection().get_selected()
        if tmp_iter != None:
            tmp_store.remove(tmp_iter)
            self.tmp_view.get_selection().select_path ((0,))

    def __add(self,type):
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return

	if rc == -4:
	    return  

        for i in self.file_dialog.get_filenames():
            if self.storefilesnotebook.get_current_page() == 0:
                model=self.exec_view.get_model()
                n=len(model)
                for k in range(n):
                    nmko=model[k][0]
                    if cmp(nmko,i)==0:
                       return 
                exec_iter = self.exec_store.append()
                self.exec_store.set_value(exec_iter, 0, i)
                self.exec_store.set_value(exec_iter, 1, type)
            elif self.storefilesnotebook.get_current_page() == 1:
                lib_iter = self.lib_store.append()
                self.lib_store.set_value(lib_iter, 0, i)
                self.lib_store.set_value(lib_iter, 1, type)
            elif self.storefilesnotebook.get_current_page() == 2:
                conf_iter = self.conf_store.append()
                self.conf_store.set_value(conf_iter, 0, i)
                self.conf_store.set_value(conf_iter, 1, type)
            elif self.storefilesnotebook.get_current_page() == 3:
                log_iter = self.log_store.append()
                self.log_store.set_value(log_iter, 0, i)
                self.log_store.set_value(log_iter, 1, type)
            elif self.storefilesnotebook.get_current_page() == 4:
                tmp_iter = self.tmp_store.append()
                self.tmp_store.set_value(tmp_iter, 0, i)
                self.tmp_store.set_value(tmp_iter, 1, type)
            else:
                iter = self.store.append()
                self.store.set_value(iter, 0, i)
                self.store.set_value(iter, 1, type)

    def exec_select(self, args):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select executable file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        #self.file_dialog.set_current_folder("/usr/sbin")
        self.file_dialog.set_current_folder("/")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.exec_entry.set_text(self.file_dialog.get_filename())

    def init_script_select(self, args):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select init script file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/etc/rc.d/init.d")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.init_script_entry.set_text(self.file_dialog.get_filename())

    def install_opt_select(self):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select install opt file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/opt")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.install_master_entry.set_text(self.file_dialog.get_filename())

    def install_usr_local_select(self):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select install /usr/local file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/usr/local")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.install_master_entry.set_text(self.file_dialog.get_filename())

    def install_home_select(self):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select install home file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/home")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.install_master_entry.set_text(self.file_dialog.get_filename())

    def install_other_select(self):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select install opt file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_current_folder("/")
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
        	return

	if rc == -4:
       		self.install_master_entry.set_text("")
		return 

        self.install_master_entry.set_text(self.file_dialog.get_filename())


    def add_file(self, args):
        self.file_dialog.set_title(_("Select file(s) that confined application creates or writes"))
        self.file_dialog.set_current_folder(self.install_master_entry.get_text())
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.file_dialog.set_select_multiple(1)
        self.__add(FILE)

    def add_dir(self, args):
        self.file_dialog.set_title(_("Select directory(s) that the confined application owns and writes into"))
        self.file_dialog.set_current_folder(self.install_master_entry.get_text())
        self.file_dialog.set_select_multiple(1)
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.__add(DIR)
        
    def on_about_clicked(self, args):
        dlg = xml.get_widget ("about_dialog")
        dlg.run ()
        dlg.hide ()

    def quit(self, args):
        gtk.main_quit()

    def setupScreen(self):
        # Bring in widgets from glade file.
        self.mainWindow = self.xml.get_widget("main_window")
        self.type = 0
        self.name_entry = self.xml.get_widget("name_entry")
        self.name_entry.connect("insert_text",self.on_name_entry_changed)
        self.name_entry.connect("focus_out_event",self.on_focus_out_event)

        self.init_script_entry = self.xml.get_widget("init_script_entry")
        self.init_script_button = self.xml.get_widget("init_script_button")
        self.install_master_entry = self.xml.get_widget("install_master_entry")
	self.rpm_radionbutton = xml.get_widget("rpm_radionbutton")
	self.rpm_entry = xml.get_widget("rpm_entry")
	
	self.install_master_button = self.xml.get_widget("install_master_button")
#       self.rpm_entry.connect("insert_text",self.rpm_name_entry_changed)
#       self.rpm_entry.connect("focus_out_event",self.rpm_focus_out_event)
        
        self.info_button = self.xml.get_widget("info_button")

        self.role_info_button = self.xml.get_widget("role_info_button")
        
        self.user_radiobutton = self.xml.get_widget("user_radiobutton")
        self.init_radiobutton = self.xml.get_widget("init_radiobutton")

        self.permissive_radiobutton = self.xml.get_widget("permissive_radiobutton")
        self.enforce_radiobutton = self.xml.get_widget("enforce_radiobutton")

        self.install_other_radiobutton = self.xml.get_widget("install_other_radiobutton")


        self.add_file_button = self.xml.get_widget("add_file_button")
        self.add_dir_button = self.xml.get_widget("add_dir_button")

        self.tmp_checkbutton = self.xml.get_widget("tmp_checkbutton")
        self.uid_checkbutton = self.xml.get_widget("uid_checkbutton")
        self.pam_checkbutton = self.xml.get_widget("pam_checkbutton")
        self.dbus_checkbutton = self.xml.get_widget("dbus_checkbutton")
        self.mail_checkbutton = self.xml.get_widget("mail_checkbutton")
        self.audit_checkbutton = self.xml.get_widget("audit_checkbutton")
        self.syslog_checkbutton = self.xml.get_widget("syslog_checkbutton")
        self.terminal_checkbutton = self.xml.get_widget("terminal_checkbutton")

        self.file_dialog = self.xml.get_widget("filechooserdialog")

#        self.view = self.xml.get_widget("write_treeview")
#        self.store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
#        self.view.set_model(self.store)
#        col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
#        col.set_resizable(True)
#        self.view.append_column(col)
#        self.view.get_selection().select_path ((0,))


        self.exec_view = self.xml.get_widget("exec_treeview")
        self.exec_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.exec_view.set_model(self.exec_store)
        exec_col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        exec_col.set_resizable(True)
        self.exec_view.append_column(exec_col)
        self.exec_view.get_selection().select_path ((0,))

        self.lib_view = self.xml.get_widget("lib_treeview")
        self.lib_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.lib_view.set_model(self.lib_store)
        lib_col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        lib_col.set_resizable(True)
        self.lib_view.append_column(lib_col)
        self.lib_view.get_selection().select_path ((0,))

        self.conf_view = self.xml.get_widget("conf_treeview")
        self.conf_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.conf_view.set_model(self.conf_store)
        conf_col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        conf_col.set_resizable(True)
        self.conf_view.append_column(conf_col)
        self.conf_view.get_selection().select_path ((0,))

        self.log_view = self.xml.get_widget("log_treeview")
        self.log_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.log_view.set_model(self.log_store)
        log_col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        log_col.set_resizable(True)
        self.log_view.append_column(log_col)
        self.log_view.get_selection().select_path ((0,))

        self.tmp_view = self.xml.get_widget("tmp_treeview")
        self.tmp_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.tmp_view.set_model(self.tmp_store)
        tmp_col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        tmp_col.set_resizable(True)
        self.tmp_view.append_column(tmp_col)
        self.tmp_view.get_selection().select_path ((0,))

	self.role_view = self.xml.get_widget("role_treeview")
	self.role_store = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING,gobject.TYPE_STRING)
	self.role_store.set_sort_column_id(1, gtk.SORT_ASCENDING)
	self.role_view.set_model(self.role_store)

	checkbox = gtk.CellRendererToggle()

        role_col = gtk.TreeViewColumn('选中', checkbox, active = 0)
        role_col.clicked()
        self.role_view.append_column(role_col)

	role_col = gtk.TreeViewColumn("角色",gtk.CellRendererText(),text=1)
	role_col.set_resizable(True)
	self.role_view.append_column(role_col)
        role_col.set_sort_column_id(1)
	
	role_col = gtk.TreeViewColumn("用户",gtk.CellRendererText(),text=2)
	role_col.set_resizable(True)
	self.role_view.append_column(role_col)
        role_col.set_sort_column_id(2)

	self.role_to_store()
        checkbox.connect("toggled", self.role_toggled)
	self.init_rpm_other_button_status()

### 设置策略生成工具加载时rpm和选择程序输入按钮的状态
    def init_rpm_other_button_status(self):
        self.install_other_radiobutton.set_active(True)
        self.install_master_entry.set_sensitive(1)
        self.rpm_entry.set_sensitive(0)
        self.rpm_scan_button.set_sensitive(0)
        self.install_master_button.set_sensitive(1)


    def output_button_clicked(self, *args):
        self.file_dialog.set_title(_("Select directory to generate policy files in"))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.file_dialog.set_select_multiple(0)
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.output_entry.set_text(self.file_dialog.get_filename())
        
    def on_name_entry_changed(self, entry, text, size, position):
        if text.find(" ") >= 0:
            entry.emit_stop_by_name("insert_text")
        
    def rpm_name_entry_changed(self, entry, text, size, position):
	if text.find("") >=0 :
	    entry.emit_stop_by_name("insert_text")
   
    def rpm_focus_out_event(self,entry,third):
	rpm_pagname = self.rpm_entry.get_text()
	if os.system('rpm -q'+" "+rpm_pagname) != 0:
		self.error(_("你必须输入或是选择一个正确的rpm包"))
		self.rpm_entry.set_text("")
		return 

    def on_focus_out_event(self, entry, third):
        name = entry.get_text()
        if self.name != name:
            if name in self.all_types:
                if self.verify(_("Type %s_t already defined in current policy.\nDo you want to continue?") % name, _("Verify Name")) == gtk.RESPONSE_NO:
                    entry.set_text("")
                    return False
            if name in self.all_modules:
                if self.verify(_("Module %s.pp already loaded in current policy.\nDo you want to continue?") % name, _("Verify Name")) == gtk.RESPONSE_NO:
                    entry.set_text("")
                    return False

#===========注释以下三行，当模块名为/etc/rc.d/init.d下的文件名时，不出现错误
#            file = "/etc/rc.d/init.d/" + name
#            if os.path.isfile(file) and self.init_script_entry.get_text() == "":
#                self.init_script_entry.set_text(file)
                
            file = "/usr/sbin/" + name
#           if os.path.isfile(file) and self.exec_entry.get_text() == "":
#               self.exec_entry.set_text(file)

        self.name = name
        return False

    def on_in_net_page_next(self, *args):
        try:
            polgen.verify_ports(self.in_tcp_entry.get_text())
            polgen.verify_ports(self.in_udp_entry.get_text())
        except ValueError, e:
            self.error(e.message)
            return True
        
    def on_out_net_page_next(self, *args):
        try:
            polgen.verify_ports(self.out_tcp_entry.get_text())
            polgen.verify_ports(self.out_udp_entry.get_text())
        except ValueError, e:
            self.error(e.message)
            return True
        
    def on_select_type_page_next(self, *args):
#       self.exec_entry.set_sensitive(self.confine_application())
#       self.exec_button.set_sensitive(self.confine_application())
        self.init_script_entry.set_sensitive(self.init_radiobutton.get_active())
        self.init_script_button.set_sensitive(self.init_radiobutton.get_active())

    def on_install_type_page_next(self, *args):
        install_type=self.install_master_entry.get_text()
        if install_type == "" and (not self.rpm_radiobutton.get_active()):
            self.error(_("请选择一个正确的程序安全路径或是rpm包"))
            return True

    def on_name_page_next(self, *args):
        name=self.name_entry.get_text()
        if not name.isalnum():
            self.error(_("您必须输入一个字母或数字组成不含空格的名字"))
            return True
        fd=os.popen('semodule -l 2>/dev/null')
        l=fd.readlines()
        fd.close()
        for i in l:
            module, ver, laki = i.split('\t')
            if cmp(name,module)==0:
                self.error(_("模块名重复，请重新输入"))
                return True
        if self.confine_application():
	   # add by hsw 
           # exe = self.exec_entry.get_text()
           # if exe == "":
           #     self.error(_("You must enter a executable"))
           #     return True
            policy=polgen.policy(name, self.get_type())
           # policy.set_program(exe)
            policy.gen_writeable()
            policy.gen_symbols()
            for f in policy.files.keys():
                iter = self.store.append()
                self.store.set_value(iter, 0, f)
                self.store.set_value(iter, 1, FILE)
                
            for f in policy.dirs.keys():
                iter = self.store.append()
                self.store.set_value(iter, 0, f)
                self.store.set_value(iter, 1, DIR)

    def stand_alone(self):
        desktopName = _("Configue SELinux")

        self.setupScreen()
        self.mainWindow.connect("destroy", self.quit)
	
        self.mainWindow.show_all()
        gtk.main()

if __name__ == "__main__":
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    if not os.path.isdir("/usr/share/sepolicy-tool/lock"):
        os.system("mkdir /usr/share/sepolicy-tool/lock")

    if os.path.isfile("/usr/share/sepolicy-tool/lock/.polgengui.lock"):
        info("策略创建工具主界面已经打开\
	或者存在/usr/share/sepolicy-tool/lock/.polgengui.lock,如果存在，请将它删除")
        exit(0)
    FD=open('/usr/share/sepolicy-tool/lock/.polgengui.lock','a')

    app = childWindow()
    app.stand_alone()
    os.system('rm -rf /usr/share/sepolicy-tool/lock/.polgengui.lock')
